package com.nisum.offertransactionservice.controller;

import com.nisum.offertransactionservice.model.EndOfSaleReq;
import com.nisum.offertransactionservice.model.OfferTransactionRequest;
import com.nisum.offertransactionservice.model.OfferTransactionResponse;
import com.nisum.offertransactionservice.service.EndOfSaleService;
import com.nisum.offertransactionservice.service.EndOfTransactionService;
import com.nisum.offertransactionservice.service.OfferCallingPEService;
import org.slf4j.MDC;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.io.IOException;
import java.util.UUID;

@RestController
public class OfferTransactionController {

    @Autowired
    private OfferCallingPEService offerCallingPEService;

    @Autowired
    private EndOfSaleService endOfSaleService;

    @Autowired
    private EndOfTransactionService endOfTransactionService;

    @PostMapping("offerTransactionCall")
    public OfferTransactionResponse getOfferTransactionResponse(@Valid @RequestBody OfferTransactionRequest offerTransactionRequest) {
        MDC.put("swy-res-unique-uuid", offerTransactionRequest.getHhId());
        MDC.put("user-id", offerTransactionRequest.getHhId());
        return offerCallingPEService.getDiscountedItemList(offerTransactionRequest);
    }


    @PostMapping("endOfSale")
    public ResponseEntity<String> endOfSale(@Valid @RequestBody EndOfSaleReq endOfSaleReq) throws IOException {
        MDC.put("swy-res-unique-uuid", endOfSaleReq.getOfferTransactionResponse().getHhid());
        MDC.put("user-id", endOfSaleReq.getOfferTransactionResponse().getHhid());
        UUID uuid = endOfSaleService.putEndofSaleResInDb(endOfSaleReq);
        return new ResponseEntity<String>(uuid.toString(), HttpStatus.OK);
    }


    @PostMapping("/endOfTransaction")
    public ResponseEntity<String> endOfTransaction(@Valid @RequestBody String uuid) {
        MDC.put("swy-res-unique-uuid", uuid);
        MDC.put("user-id", uuid);
        String response = endOfTransactionService.endOfTransaction(uuid);
        return new ResponseEntity(response, HttpStatus.OK);
    }

}



