package com.nisum.offertransactionservice.service;

import com.nisum.offertransactionservice.converter.OfferConvertion;
import com.nisum.offertransactionservice.dao.OfferLookupRepo;
import com.nisum.offertransactionservice.genericexception.GlobalApiGenericException;
import com.nisum.offertransactionservice.model.*;
import com.safeway.epe.model.Response;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.PropertySource;
import org.springframework.http.*;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.client.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import javax.annotation.PostConstruct;
import java.util.ArrayList;
import java.util.List;

@Service
@PropertySource("classpath:application.properties")
@Slf4j
public class OfferCallingPEService {


    @Autowired
    private OfferLookupRepo offerLookupRepo;

   /* @Autowired
    private PEResponseToOfferTransactionResConverter peResponseToOfferTransactionResConverter;*/

    private WebClient webClient;

    @Value("${promotion.engine.calculate.discount.url}")
    private String promotionEngineUrl;

    @Value("${promotion.engine.baseUrl}")
    private String baseUrl;


    @PostConstruct
    public void init() {
        webClient = WebClient.builder().baseUrl(baseUrl).
                defaultHeader(HttpHeaders.CONTENT_TYPE, MediaType.APPLICATION_JSON_VALUE).
                filter(logRequest()).
                filter(logResponse()).
                build();
    }


    private ExchangeFilterFunction logRequest() {
        return ExchangeFilterFunction.ofRequestProcessor(clientRequest -> {
            log.info("Promotional Request {}", clientRequest);
            return Mono.just(clientRequest);
        });
    }

    private ExchangeFilterFunction logResponse() {
        return ExchangeFilterFunction.ofResponseProcessor(clientResponse -> {
            log.info("Promotional Request {}", clientResponse);
            return Mono.just(clientResponse);
        });
    }

    public OfferTransactionResponse getDiscountedItemList(OfferTransactionRequest offerTransactionRequest) {
        log.info("Inside getDiscountedItemList Method");
        List<OfferLookup> eligibleOffer = new ArrayList();
        offerLookupRepo.findAll().forEach(eligibleOffer::add);
        if (eligibleOffer.isEmpty()) {
            log.error("Offer lookup Object is empty");
            throw new GlobalApiGenericException("Unable to get the Offer Response ", new Exception("exception e"), "400", false);
        }
        log.debug("Offer lookup Object {}", offerLookupRepo);
        OfferTransactionResponse offerTransactionResponse;
        PERequest peRequest = OfferConvertion.INSTANCE.map(offerTransactionRequest);
        peRequest.setEligibleOffers(eligibleOffer);
        log.info("Promotional Engine WebClient call Start");
        Flux<PEResponse> peResponseFlux = getPeResponseFlux(peRequest);
        log.debug("Promotional Engine WebClient call End");
        offerTransactionResponse = OfferConvertion.INSTANCE.map(peResponseFlux.toStream().findFirst().get());
        log.debug("Offer Transaction Response {}", offerTransactionResponse);
        return offerTransactionResponse;
    }

    public Flux<PEResponse> getPeResponseFlux(PERequest peRequest) {
        return webClient.post().
                    uri(promotionEngineUrl).
                    accept(MediaType.APPLICATION_JSON).
                    contentType(MediaType.APPLICATION_JSON).
                    body(Mono.just(peRequest), PERequest.class).
                    retrieve().
                    onStatus(HttpStatus::is4xxClientError, clientResponse -> handleError(clientResponse)).
                    onStatus(HttpStatus::is5xxServerError, clientResponse -> handleError(clientResponse)).
                    bodyToFlux(PEResponse.class);
    }

    private Mono<? extends Throwable> handleError(ClientResponse clientResponse) throws GlobalApiGenericException {
        final Mono<Response> responseMono = clientResponse.bodyToMono(Response.class);
        return responseMono.flatMap(response -> {
            log.error("Error Response While Calling PE Service {}", response);
            throw new GlobalApiGenericException(response, response.getHttpStatusCode());
        });
    }
}
